#!/usr/bin/env python3

import sys
import os
import argparse
from datetime import datetime

if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))

from apsdk import env
from apcommon import iniparser
from apbootstrap_autopilot_ini import Apbootstrap_Autopilot_Ini
from apbootstrap_apt_proxy import Apbootstrap_Apt_Proxy
from apcommon import partitioning

class Bootstrap(object):
    def __init__(self, apbootstrap_ini_path="", linux_bootstrap_ini_path="", autopilot_ini_path="",
                 apt_proxy_file="", apt_proxy_server="", sections_to_reset=[]):

        # Variables passed in during initialization
        self.apbootstrap_ini_path = apbootstrap_ini_path
        self.linux_bootstrap_ini_path = linux_bootstrap_ini_path
        self.autopilot_ini_path = autopilot_ini_path
        self.apt_proxy_file = apt_proxy_file
        self.apt_proxy_server = apt_proxy_server
        self.sections_to_reset = sections_to_reset

        # Initialize all objects and parsers so we have them available right away
        self.apbootstrap_ini_parser_object = iniparser.INIParser(self.apbootstrap_ini_path)
        self.apbootstrap_autopilot_ini = Apbootstrap_Autopilot_Ini (
            self.linux_bootstrap_ini_path,self.autopilot_ini_path)
        self.apbootstrap_apt_proxy = Apbootstrap_Apt_Proxy(self.apt_proxy_file,self.apt_proxy_server)

        # Only write changes to apbootstrap.ini if the contents change
        self.changed_apbootstrap_ini = False

    # Generic abstraction function to check if an ini file exists on file system
    def __check_ini_exists(self,ini_path):
        if (os.path.isfile(ini_path)):
            return True
        else:
            return False

    # Generic abstraction function to create a fresh ini file in memory
    def __create_ini(self,ini_object):
        if (not ini_object.create()):
            return False

        return True

    # Generic abstraction function to call read on any iniparser object
    def __read_ini(self,ini_object):
        if (ini_object.read()):
            return True
        else:
            return False

    # Generic abstraction function to call write on any iniparser object
    def __write_ini(self,ini_object):
        if(not ini_object.write()):
            return False

        return True

    # When creating a new apbootstrap.ini file in memory, pre-populate it with default values
    # It needs to know what to execute right away on first run
    def __populate_new_apbootstrap_ini(self):
        if (not self.apbootstrap_ini_parser_object.add_section("autopilot_ini")):
            return False
        if (not self.apbootstrap_ini_parser_object.set("autopilot_ini", "Execute", "True")):
            return False

        if (not self.apbootstrap_ini_parser_object.add_section("apt-proxy")):
            return False
        if (not self.apbootstrap_ini_parser_object.set("apt-proxy", "Execute", "True")):
            return False

        # if (not self.apbootstrap_ini_parser_object.add_section("Partitioning")):
        #    return False
        # if (not self.apbootstrap_ini_parser_object.set("Partitioning","Execute","True")):
        #    return False

        return True

    # If apbootstrap.ini needs to re-run any sections due to updates, etc.  This will instruct it to reset certain
    # sections to be run again next time it executes
    # This is done via commandline using option -r (see bottom of file)
    def __reset_apbootstrap_ini_sections(self):
        for section in self.sections_to_reset:
            if (not self.apbootstrap_ini_parser_object.set(section, "Execute", "True")):
                return False

        return True

    # Function that determines whether to create a new apbootstrap.ini if one doesn't exist or to use an
    # existing one if it's being changed
    def __configure_apbootstrap_ini(self):
        if (not self.__check_ini_exists(self.apbootstrap_ini_path)):
            print("apbootstrap: apbootstrap.ini does not exist; must create a new one")
            if (not self.__create_ini(self.apbootstrap_ini_parser_object)):
                print("apbootstrap: Could not create new apbootstrap ini file (apbootstrap_ini_parser_object.create_ini)")
                return False

            if (not self.__populate_new_apbootstrap_ini()):
                print("apbootstrap: Could not populate new apbootstrap ini file ("
                      "apbootstrap_ini_parser_object.populate_new_apbootstrap_ini)")
                return False

            return True
        else:
            if (not self.__read_ini(self.apbootstrap_ini_parser_object)):
                print("apbootstrap: Could not read apbootstrap ini file (apbootstrap_ini_parser_object.read_ini)")
                return False

            return True

    # Lets apbootstrap know if autopilot.ini actions have been completed by setting them in apbootstrap.ini so it
    # doesn't run them again
    def __set_autopilot_executed(self):
        if (not self.apbootstrap_ini_parser_object.remove_section("autopilot_ini")):
            return False

        if (not self.apbootstrap_ini_parser_object.add_section("autopilot_ini")):
            return False

        if (not self.apbootstrap_ini_parser_object.set("autopilot_ini", "Execute", "False")):
            return False

        if (not self.apbootstrap_ini_parser_object.set("autopilot_ini", "Success", "True")):
            return False

        if (not self.apbootstrap_ini_parser_object.set("autopilot_ini", "LastRun", datetime.now().strftime("%Y-%m-%d %H:%M:%S"))):
            return False

        self.changed_apbootstrap_ini = True

        return True

    # Lets apbootstrap know if apt-proxy actions have been completed by setting them in apbootstrap.ini so it
    # doesn't run them again
    def __set_apt_proxy_executed(self):
        if (not self.apbootstrap_ini_parser_object.remove_section("apt-proxy")):
            return False

        if (not self.apbootstrap_ini_parser_object.add_section("apt-proxy")):
            return False

        if (not self.apbootstrap_ini_parser_object.set("apt-proxy", "Execute", "False")):
            return False

        if (not self.apbootstrap_ini_parser_object.set("apt-proxy", "Success", "True")):
            return False

        if (not self.apbootstrap_ini_parser_object.set("apt-proxy", "LastRun", datetime.now().strftime("%Y-%m-%d %H:%M:%S"))):
            return False

        self.changed_apbootstrap_ini = True

        return True

    # def partition_disks(self):
    #     if (self.apbootstrap_ini_parser_object.getboolean("Partitioning","Execute")):
    #         if (self.apbootstrap_ini_parser_object.get("Partitioning","Type")):
    #             self.partition_type = self.apbootstrap_ini_parser_object.get("Partitioning","Type")
    #
    #             self.partition = partitioning.Partitioning(self.partition_type)
    #             self.partition.main()

    # def set_partitioning_executed(self):
    #     self.apbootstrap_ini_parser_object.remove_section("Partitioning")
    #     self.apbootstrap_ini_parser_object.add_section("Partitioning")
    #     self.apbootstrap_ini_parser_object.set("Partitioning","Execute","False")
    #     self.apbootstrap_ini_parser_object.set("Partitioning","Success","True")
    #     self.apbootstrap_ini_parser_object.set("Partitioning","LastRun",datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
    #     self.apbootstrap_ini_parser_object.set("Partitioning","Type",self.partition.get_disk_config_type())
    #
    #     block_devices = self.partition.get_block_devices()
    #
    #     for device,label in sorted(block_devices.items()):
    #         self.apbootstrap_ini_parser_object.set("Partitioning",device,label)

    def main(self):
        # If we're resetting sections that need to be run again, use this path
        # If the ini file is not generated yet on initial run we do not run this first but instead create
        # the ini file and run it normally since it will already re-run everything
        # The reset is redundant in this case
        if (self.sections_to_reset and self.__check_ini_exists(self.apbootstrap_ini_path)):
            print("Sections to reset:")
            print(sections_to_reset)

            if (not self.__check_ini_exists(self.apbootstrap_ini_path)):
                print("apbootstrap: Could not find apbootstrap ini file (apbootstrap_ini_parser_object.check_ini_exists)")
                sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error

            if (not self.__read_ini(self.apbootstrap_ini_parser_object)):
                print("apbootstrap: Could not read apbootstrap ini file (apbootstrap_ini_parser_object.read_ini)")
                sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error

            if (not self.__reset_apbootstrap_ini_sections()):
                print("apbootstrap: Could not reset apbootstrap sections (apbootstrap_ini_parser_object.reset_apbootstrap_ini_sections)")
                sys.exit(os.EX_SOFTWARE)  # EX_SOFTWARE = an internal software error was detected

            if (not self.__write_ini(self.apbootstrap_ini_parser_object)):
                print("apbootstrap: Could not write apbootstrap ini file (apbootstrap_ini_parser_object.write_ini)")
                sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error

            sys.exit(os.EX_OK)  # EX_OK = everything executed normally

        # For normal operation use this path which runs through configuration if necessary
        else:
            if (not self.__configure_apbootstrap_ini()):
                print("apbootstrap: Could not configure apbootstrap.ini file (apbootstrap_ini_parser_object.configure_apbootstrap_ini)")
                sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error

            print("apbootstrap: Should configure_autopilot_ini be executed?  Checking abbootstrap config:")
            if (self.apbootstrap_ini_parser_object.getboolean("autopilot_ini", "Execute")):
                if (not self.apbootstrap_autopilot_ini.configure_autopilot_ini()):
                    print("apbootstrap: Could not configure autopilot.ini file (apbootstrap_autopilot_ini.configure_autopilot_ini)")
                    sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error
                else:
                    if (not self.__set_autopilot_executed()):
                        print("apbootstrap: Could not configure autopilot.ini file (apbootstrap_ini_parser_object.configure_autopilot_ini)")
                        sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error

            print("apbootstrap: Should configure_apt_proxy be executed?  Checking abbootstrap config:")
            if (self.apbootstrap_ini_parser_object.getboolean("apt-proxy", "Execute")):
                if (not self.apbootstrap_apt_proxy.configure_apt_proxy()):
                    print("apbootstrap: Could not configure apt proxy file (apbootstrap_apt_proxy.configure_apt_proxy)")
                    sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error
                else:
                    if (not self.__set_apt_proxy_executed()):
                        print("apbootstrap: Could not set apt-proxy executed in apbootstrap.ini (apbootstrap_ini_parser_object.set_apt_proxy_executed)")
                        sys.exit(os.EX_OSFILE)  # EX_OSFILE = some system file did not exist, could not be opened, or had some other kind of error

            # self.partition_disks()

            if (self.changed_apbootstrap_ini):
                self.__write_ini(self.apbootstrap_ini_parser_object)

            sys.exit(os.EX_OK)  # EX_OK = everything executed normally

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="Send a command to apbootstrap service.")
    # For resetting sections, run apbootstrap.py with -r "section1,section2,etc"
    # then re-run it again with no options to execute changes and set things back to Execute:False
    parser.add_argument("-r", "--reset", metavar="SECTION", help="section or sections to reset so they re-run next "
                                                                 "time apbootstrap runs; if supplying multiple "
                                                                 "sections please use commas to separate them and "
                                                                 "wrap them in double quotes")

    args = parser.parse_args()
    if (args.reset):
        sections_to_reset = args.reset.split(",")
    else:
        sections_to_reset = []

    apbootstrap_ini_path = env.get_data_root() + "/apbootstrap/apbootstrap.ini"
    linux_bootstrap_ini_path = env.get_linux_bootstrap_ini_path()
    autopilot_ini_path = env.get_autopilot_ini_path()
    apt_proxy_file = "/etc/apt/apt.conf.d/02proxy"
    apt_proxy_server = "http://aplinuxservices.trafficmanager.net:3142"

    Bootstrap(apbootstrap_ini_path, linux_bootstrap_ini_path, autopilot_ini_path, apt_proxy_file, apt_proxy_server,
              sections_to_reset).main()